/*
 * Copyright (C) 1997-2004 Andre Noll <noll@mathematik.tu-darmstadt.de>
 * 
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 * 
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 */

#include "para.h"
#include "server.cmdline.h"

/* afs status flags */
#define AFS_NOMORE 1
#define AFS_NEXT 2
#define AFS_REPOS 4
#define AFS_PLAYING 8
#define AFS_REREAD_DB_INFO 16

/* contains a list of all valid commands */
struct linked_cmd_list {
        struct linked_cmd_list *next;
	struct command *cmd;
};

/* data needed to authenticate user */
struct user{
	char name[MAXLINE];
	char pubkey_file[_POSIX_PATH_MAX];
	unsigned int perms;
};

/* contains data for arbitrary command */
struct command {
	char name[MAXLINE];
	unsigned int perms, modus;
	char description[MAXLINE];
	char usage[MAXLINE];
	char help[STRINGSIZE];
	int (*handler)(int, int, char **);
};


/* there's one struct of this type which lives in shared memory
 * for communication between the server instances. There are two reasons
 * for a variable to be inlcuded here:
 * (a) at least one command (i.e. child of the server) must be able to 
 *     change its value
 * or
 * (b) The contents are listed in the stat command and have to be up to date  
 */
#define MMD_DBINFO_SIZE 16384
struct mp3_meta_data{
/* afs */ 
	long unsigned int  total, size;
	int afs_status_flags;
	mp3info mp3;
	char filename[_POSIX_PATH_MAX];
	char mtime[MAXLINE];
/* server */
	unsigned int num_played;
	unsigned int num_commands;
	unsigned int active_connections;
	pid_t server_pid;
/* dbtool */
	int score;
	char dbinfo[MMD_DBINFO_SIZE];
	int dbtool_has_pic; /* FIXME */
	int stream, old_stream;

	struct linked_cmd_list *lcl_head; /* FIXME */
	char **stream_list; /* FIXME */
/* configuration */
	char config_file[_POSIX_PATH_MAX]; /* FIXME */
	char user_list[_POSIX_PATH_MAX]; /*FIXME */ 
	char logfile[_POSIX_PATH_MAX];  /*FIXME */ 
};

int update_dbinfo(void);
void afs_send_chunk(int);
void afs_status_tohuman(int, char*);
void afs_preselect(void);
FILE *get_stream_writer_pipe(char *);

void uptime_str(int , char *);
unsigned int num_connects(int);
int server_uptime (int );
void clog(int, char* ,...);
//void sigint_sigterm_handler(int s) ;
void cmd_perms_itohuman(unsigned int, char*);
void reload_log(void);
int check_perms(unsigned int, struct linked_cmd_list*);

struct linked_cmd_list *parse_cmd(char*);
void lcl_init();
struct linked_cmd_list *lcl_find(char*);
void lcl_nuke(void);

void mmd_unlock(void);
void mmd_lock(void);
void mmd_dup(struct mp3_meta_data *);
int find_stream(char *, struct mp3_meta_data *);
